----------------------------------------------------------------------------------
-- Company:	FRI
-- Engineer: Miha Moskon
-- 
-- Create Date:    13:16:40 10/03/2007 
-- Design Name: 
-- Module Name:    lcd_config - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd_config is
	port(
		CLK_in	: in	STD_LOGIC;		-- ura
		RST_in	: in	STD_LOGIC;		-- skupni reset signal
		
		LCD_E_out	: out	STD_LOGIC;		-- lcd enable 
		LCD_RS_out	: out	STD_LOGIC;	-- lcd reg. select
		LCD_RW_out	: out	STD_LOGIC;	-- lcd read/write
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
		
		CONFIG_IN_PROG_in	: in	STD_LOGIC;	-- odvija se konfiguracija
		CONFIG_DONE_out	: out	STD_LOGIC);		-- konfiguracija je koncana

end lcd_config;

architecture Behavioral of lcd_config is
	component counter
		generic(
			size : integer := 8);
		port(
			CLK_in	: IN STD_LOGIC;
			WE_in	: IN STD_LOGIC;
			VALUE_in	: IN STD_LOGIC_VECTOR(size-1 downto 0);
			COUNT_UP_out : OUT STD_LOGIC);		
	end component;
	
	component lcd_send
	port(
		CLK_in	: in	STD_LOGIC;
		RST_in	: in	STD_LOGIC;		
		LCD_E_out	: out	STD_LOGIC;
		LCD_RS_out	: out	STD_LOGIC;
		LCD_RW_out	: out	STD_LOGIC;
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);		
		DATA_in	:	in STD_LOGIC_VECTOR(9 downto 0);			
		SEND_IN_PROG_in	: in	STD_LOGIC;
		SEND_DONE_out	: out	STD_LOGIC);			
	end component;
	
	
	-- stanja config avtomata
	type STATES is (DUMMY, FSET, EMS, D_ONOFF, CLR, INIT_DELAY, DELAY, SET_ADDR, DONE);	-- definiranje moznih stanj avtomata
	signal currSt, nextSt	:	STATES;		-- trenutno in naslednje stanje
	
	-- signali za stevec
	signal WE_counter, count_up	: STD_LOGIC;
	signal value_to_counter	: STD_LOGIC_VECTOR(19 downto 0);		-- steti mora znati do 82.000 (17 bitov)
	
	-- signali za lcd_send modul
	signal LCD_E_SEND	: STD_LOGIC;
	signal LCD_RS_SEND	: STD_LOGIC;
	signal LCD_RW_SEND	: STD_LOGIC;
	signal SF_D_SEND	: STD_LOGIC_VECTOR(11 downto 8);
	signal DATA_SEND	: STD_LOGIC_VECTOR(9 downto 0);
	signal send_in_prog	: STD_LOGIC;
	signal send_done	: STD_LOGIC;
	
	-- pomozni signali, ki jih povezem z izhodi
	signal LCD_E	: STD_LOGIC;		-- lcd enable 
	signal LCD_RS	: STD_LOGIC;	-- lcd reg. select
	signal LCD_RW	: STD_LOGIC;	-- lcd read/write
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
	signal CONFIG_DONE	: STD_LOGIC;		-- konfiguracija je koncana
	
begin
	
-- IO ASSIGNMENT
	LCD_E_out	<= LCD_E;
	LCD_RS_out	<= LCD_RS;
	LCD_RW_out	<= LCD_RW;
	SF_D_inout	<= SF_D;
	CONFIG_DONE_out	<= CONFIG_DONE;
	
	
--------------------------
-- INSTANCIRANJE KOMPONENT
--------------------------

	-- instanca stevca
	counter20bit:counter
		generic map
		( 
			size => 20		-- 17-bitni stevec: znati mora steti do 82.000
		)
		port map
		(
			CLK_in	=> CLK_in,
			WE_in	=> WE_counter,
			VALUE_in	=> value_to_counter,
			COUNT_UP_out	=> count_up		
		);
		
	-- instanca lcd_send avtomata
	lcd_send_logic:lcd_send
	port map
	(
		CLK_in	=> CLK_in,
		RST_in	=> RST_in,
		LCD_E_out	=>	LCD_E_SEND,
		LCD_RS_out	=> LCD_RS_SEND,
		LCD_RW_out	=> LCD_RW_SEND,
		SF_D_inout	=>	SF_D_SEND,
		DATA_in	=>	DATA_SEND,
		SEND_IN_PROG_in	=> send_in_prog,
		SEND_DONE_out	=> send_done
	);

--------------------
-- CONFIGURE AVTOMAT
--------------------

	-- sinhroni proces
	sync_proc:process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if ( (RST_in = '1') OR (CONFIG_IN_PROG_in = '0') ) then 
				currSt	<= DUMMY;
			else
				currSt	<= nextSt;
			end if;
		end if;
	end process;
	
	
	-- asinhroni proces
	config_proc: process (currSt, CONFIG_IN_PROG_in, count_up, send_done)
	begin
		-- lcd		
		LCD_E	<= '0';
		LCD_RS	<= '0';
		LCD_RW	<= '0';
		SF_D	<= "0000";
		
		-- stevec
		WE_counter <= '0';
		value_to_counter <= (others => '0');
		
		-- vhod v lcd_top modul
		CONFIG_DONE <= '0';
		
		-- vhodi v lcd_send modul
		send_in_prog	<= '0';
		DATA_SEND	<= (others => '0');
		
		case currSt is
		
		-- DUMMY - cakanje na zacetek konfiguracije
		
			-- dummy state (v njem je avtomat, ko je config_in_prog = 0)
			when DUMMY	=>
				if (CONFIG_IN_PROG_in = '0') then
					nextSt <= DUMMY;					
				else
					nextSt <= FSET;			
				end if;
		
			-- FUNCTION SET UKAZ
			when FSET	=>
				-- vhodi v lcd_send modul
				DATA_SEND	<= "0000101000";
				send_in_prog	<= '1';
								
				-- izhodi iz lcd_send modula
				LCD_E	<= LCD_E_SEND;
				LCD_RS	<= LCD_RS_SEND;
				LCD_RW	<= LCD_RW_SEND;
				SF_D	<= SF_D_SEND;
				
				if (send_done = '0')	then
					nextSt	<= FSET;
				else
					nextSt	<= EMS;
				end if;
				
				
			-- ENTRY MODE SET UKAZ
			when EMS	=>
				-- vhodi v lcd_send modul
				DATA_SEND	<= "0000000110";
				send_in_prog	<= '1';
								
				-- izhodi iz lcd_send modula
				LCD_E	<= LCD_E_SEND;
				LCD_RS	<= LCD_RS_SEND;
				LCD_RW	<= LCD_RW_SEND;
				SF_D	<= SF_D_SEND;
				
				if (send_done = '0')	then
					nextSt	<= EMS;
				else
					nextSt	<= D_ONOFF;
				end if;
				
			-- DISPLAY ON/OFF UKAZ
			when D_ONOFF	=>
				-- vhodi v lcd_send modul
				DATA_SEND	<= "0000001111";
				send_in_prog	<= '1';
								
				-- izhodi iz lcd_send modula
				LCD_E	<= LCD_E_SEND;
				LCD_RS	<= LCD_RS_SEND;
				LCD_RW	<= LCD_RW_SEND;
				SF_D	<= SF_D_SEND;
				
				if (send_done = '0')	then
					nextSt	<= D_ONOFF;
				else
					nextSt	<= CLR;
				end if;
				
			-- CLEAR DISPLAY UKAZ
			when CLR	=>
				-- vhodi v lcd_send modul
				DATA_SEND	<= "0000000001";
				send_in_prog	<= '1';
								
				-- izhodi iz lcd_send modula
				LCD_E	<= LCD_E_SEND;
				LCD_RS	<= LCD_RS_SEND;
				LCD_RW	<= LCD_RW_SEND;
				SF_D	<= SF_D_SEND;
				
				if (send_done = '0')	then
					nextSt	<= CLR;
				else
					nextSt	<= INIT_DELAY;
				end if;
				
			-- INICIALIZACIJA ZAKASNITVE ZA CLEAR DISPLAY UKAZOM
			when INIT_DELAY	=>
				WE_counter	<= '1';
				value_to_counter	<= x"14050";		-- 82000 urinih period (1,64ms)
				
				nextSt	<= DELAY;
				
			-- ZAKASNITEV ZA CLEAR DISPLAY UKAZOM
			when DELAY	=>
				if (count_up = '0') then
					nextSt	<= DELAY;
				else
					nextSt	<=	SET_ADDR;
				end if;
		
			
			-- SET DD RAM ADDRESS - ne spada v osnovno konfiguracijo, ampak ga je potrebno nastaviti pred posiljanjem
			when SET_ADDR	=>
				-- vhodi v lcd_send modul
				DATA_SEND	<= "0010000000";
				send_in_prog	<= '1';
								
				-- izhodi iz lcd_send modula
				LCD_E	<= LCD_E_SEND;
				LCD_RS	<= LCD_RS_SEND;
				LCD_RW	<= LCD_RW_SEND;
				SF_D	<= SF_D_SEND;
				
				if (send_done = '0')	then
					nextSt	<= SET_ADDR;
				else
					nextSt	<= DONE;
				end if;
				
			-- DONE		
			when DONE	=>
				CONFIG_DONE <= '1';
				
				nextSt	<= DUMMY;
	
		end case;
	
	end process;


end Behavioral;

